package depesz::checkMail;

=head1 NAME

depesz::checkMail - module which checks for new mails (mail count) in mbox-style mail folders

=head1 SYNOPSIS

This module does the job for dCheckMail program.

=head1 DESCRIPTION

=cut

use strict;
use Config::IniFiles;
use File::Find;

$| = 1;
$depesz::checkMail::VERSION=1.0;
$depesz::checkMail::REAL_VERSION = $depesz::checkMail::VERSION.' rev. '.(qw$Revision: 1.1 $)[-1];
$depesz::checkMail::TEMP_FILES = [];

=item B<new>

Main function - constructor of depesz::checkMail object.

The only parameter it can get it program's name. In case when no paramaeteres are suplied 'dCheckMail' is used.

=cut

sub new {
	my $class = shift;
	my $self = {};
	
	bless $self, $class;
	$self->{'_NAME'} = shift || 'dCheckMail';
	
	return $self;
}


=item B<go>

Method which starts object's work: reading configuration, finding and parsing mailboxes, printing results.

=cut

sub go {
	my $self = shift;
	unless ($self->readConfig()) {
		print STDERR "Error while reading configuration file(s):\n" . $self->{'_ERROR'};
		return undef;
	}
	
	$self->{'OUTPUT'} = [];
	
	for my $rFolder (@{$self->{'CFG'}->{'FOLDERS'}}) {
		$self->processFolder($rFolder);
	}

	for my $rItem (@{$self->{'OUTPUT'}}) {
		my $sSkip = (defined($rItem->{'Folder'}->{'SKIPZERO'}) ? $rItem->{'Folder'}->{'SKIPZERO'} : $self->{'CFG'}->{'GLOBAL'}->{'SKIPZERO'});
	
		next if $sSkip && !$rItem->{'Unread'};
		
		my $sFormat = $rItem->{'Folder'}->{'FORMAT'} || $self->{'CFG'}->{'GLOBAL'}->{'FORMAT'};
		my $sFileNameRemover = $rItem->{'Folder'}->{'REMOVER'} || $self->{'CFG'}->{'GLOBAL'}->{'REMOVER'};
		$sFileNameRemover =~ s/\$\{HOME\}/$ENV{'HOME'}/g if $sFileNameRemover;
		my $sFileName = $rItem->{'File'};
		if ($sFileNameRemover) {
			$sFileName =~ s/$sFileNameRemover//;
		}
		$sFormat =~ s/\$(-?\d*)\{UNREAD\}/sprintf("%$1u", $rItem->{'Unread'})/ge;
		$sFormat =~ s/\$(-?\d*)\{COUNT\}/sprintf("%$1u", $rItem->{'Count'})/ge;
		$sFormat =~ s/\$(-?\d*)\{FOLDER\}/sprintf("%$1s", $rItem->{'Folder'}->{'NAME'})/ge;
		$sFormat =~ s/\$(-?\d*)\{NAME\}/sprintf("%$1s", $sFileName)/ge;
		print $sFormat."\n";
	}
	
	return 1;
}



=item B<processFolder>

Parses mbox, and gets all needed information from it.

=cut

sub processFolder {
	my $self = shift;
	my $rFolder = shift;
	
	my $rFiles = $self->getFileNames($rFolder);

	if (scalar(@$rFiles) == 0) {
		if ($self->{'CFG'}->{'GLOBAL'}->{'verbose'}) {
			print STDERR 'Folder: '.$rFolder->{'NAME'}.' does not contain any files!!!';
		}
		return;
	}
	
	for (my $i = 0; $i <= $#{$rFiles}; $i++) {
		if ($self->{'CFG'}->{'GLOBAL'}->{'verbose'}) {
			printf("\015%u/%u", $i, $#{$rFiles});
		}
		$self->processFile($rFolder, $rFiles->[$i]);
	}
	if ($self->{'CFG'}->{'GLOBAL'}->{'verbose'}) {
		print ""x(1+2*length($#{$rFiles}));
	}
}


=item B<processFile>

Here is done the real work with mbox-files.

=cut

sub processFile {
	my $self = shift;
	my $rFolder = shift;
	my $sFileName = shift;

	return if -z $sFileName;
	
	my $sCount = 0;
	my $sUnreadCount = 0;
	my $sActive = "";
	my $sInHead = 0;
	my $sLinesToSkip=0;

	open(IN, "<", $sFileName) || die "Strange error while opening $sFileName!!!";

	while (<IN>) {
		if (/^From\s/) {
			$sActive=$_;
			$sInHead = 1;
			$sCount++;
			$sLinesToSkip=0;
			next;
		}
		next unless $sInHead;
		$sActive.=$_;
		if (/^Lines:\s*(\d+)\s*$/) {
			$sLinesToSkip=$1;
			next;
		}
		if (/^\s*$/) {
			$sInHead = 0;
			if ($sLinesToSkip) {
				<IN> for 1..$sLinesToSkip;
			}
			if ($sActive !~ /^Status:.*R/m) {
				$sUnreadCount++;
			}
			next;
		}
	}

	push @{$self->{'OUTPUT'}}, {
		'Folder'	=>	$rFolder,
		'File'		=>	$sFileName,
		'Count'		=>	$sCount,
		'Unread'	=>	$sUnreadCount
	};
	
	close IN;
}


=item B<getFileNames>

Get's filenames which are inside this "mbox"

=cut

sub getFileNames {
	my $self = shift;
	my $rFolder = shift;
	my @aReply = ();

	if (defined($rFolder->{'File'})) {
		if (ref($rFolder->{'File'})) {
			push @aReply, grep {-f && -r} map {s/\$\{HOME\}/$ENV{'HOME'}/g; $_} @{$rFolder->{'File'}};
		} else {
			push @aReply, grep {-f && -r} map {s/\$\{HOME\}/$ENV{'HOME'}/g; $_} ($rFolder->{'File'});
		}
	}
	if (defined($rFolder->{'Dir'})) {
		my @aDirs = ();
		if (ref($rFolder->{'Dir'})) {
			push @aDirs, grep {-d && -r} map {s/\$\{HOME\}/$ENV{'HOME'}/g; $_} @{$rFolder->{'Dir'}};
		} else {
			push @aDirs, grep {-d && -r} map {s/\$\{HOME\}/$ENV{'HOME'}/g; $_} ($rFolder->{'Dir'});
		}
		$depesz::checkMail::TEMP_FILES = [];
		find(\&addNewFile, @aDirs);
		push @aReply, @{$depesz::checkMail::TEMP_FILES};
	}
	return \@aReply;
}


=item B<addNewFile>

Slave function for File::Find module

=cut

sub addNewFile {
	push @{$depesz::checkMail::TEMP_FILES}, $File::Find::name if -f && -r;
}

=item B<readConfig>

Method which reads config files and merges them (main /etc/ds/dCheckMail.ini and local $CONFIG_DIR/ds/dCheckMail.ini)

=cut

sub readConfig {
	my $self = shift;
	
	$self->{'CFG'}->{'GLOBAL'}->{'verbose'} = 0;
	
	$self->{'CFG'}->{'GLOBAL'}->{'FORMAT'} = '$3{UNREAD}/$-3{COUNT} ${FOLDER}:${NAME}';
	
	$self->{'CFG'}->{'GLOBAL'}->{'SKIPZERO'} = 0;
	
	my $sTopLevelConfigFileName = '/etc/ds/'.$self->{'_NAME'}.'.ini';

	my $sLocalConfigFileName = (
		$ENV{'CONFIG_DIR'} ?
			$ENV{'CONFIG_DIR'} . '/ds/' . $self->{'_NAME'} . '.ini' :
			(
				$ENV{'HOME'} ?
				$ENV{'HOME'}.'/.ds/'.$self->{'_NAME'} . '.ini' :
				'/non/existent'
		)
	);

	my $sConfigOK = 0;
	
	if (
		(-f $sTopLevelConfigFileName) &&
		(-r $sTopLevelConfigFileName)
	) {
		$self->{'_CFG'}->{'_TOP'} = new Config::IniFiles( '-file' => $sTopLevelConfigFileName );
		$sConfigOK += 1;
	}
		
	if (
		(-f $sLocalConfigFileName) &&
		(-r $sLocalConfigFileName)
	) {
		$self->{'_CFG'}->{'_LOCAL'} = new Config::IniFiles( '-file' => $sLocalConfigFileName );
		$sConfigOK += 2;
	}
	
	unless ($sConfigOK) {
		$self->{'_ERROR'} = 'Config files ('.$sTopLevelConfigFileName.' or '.$sLocalConfigFileName.') couldn\'t be found!';
		return undef;
	}
	
	my @aConfigi;

	push @aConfigi, $self->{'_CFG'}->{'_TOP'} if 1 & $sConfigOK;
	push @aConfigi, $self->{'_CFG'}->{'_LOCAL'} if 2 & $sConfigOK;
	
	for my $rCFG (@aConfigi) {
		for my $sSect ($rCFG->Sections()) {
			if ($sSect eq 'GLOBAL') {
				for my $sParam ($rCFG->Parameters($sSect)) {
					my @aTemp = $rCFG->val($sSect, $sParam);
					$self->{'CFG'}->{'GLOBAL'}->{$sParam} = (scalar(@aTemp) > 1 ? [@aTemp] : $aTemp[0]);
				}
			} else {
				my $rTemp = {};
				if ($self->{'CFG'}->{'SECTS'}->{$sSect}) {
					$rTemp = $self->{'CFG'}->{'SECTS'}->{$sSect}
				} else {
					$rTemp->{'NAME'} = $sSect;
					push @{$self->{'CFG'}->{'FOLDERS'}}, $rTemp;
					$self->{'CFG'}->{'SECTS'}->{$sSect} = $rTemp;
				}
				for my $sParam ($rCFG->Parameters($sSect)) {
					my @aTemp = $rCFG->val($sSect, $sParam);
					$rTemp->{$sParam} = (scalar(@aTemp) > 1 ? [@aTemp] : $aTemp[0]);
				}
			}
		}
	}

	return 1;
}

=head1 AUTHOR

hubert depesz lubaczewski E<lt>depesz@depesz.plE<gt>

=head1 LICENSE

  This module is free software; you can redistribute it and/or modify it
  under the terms of either:

  a) the GNU General Public License as published by the Free Software
  Foundation; either version 1, or (at your option) any later version,
  
  or

  b) the "Artistic License" which comes with this module.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See either
  the GNU General Public License or the Artistic License for more details.

  You should have received a copy of the Artistic License with this
  module, in the file ARTISTIC.  If not, I'll be glad to provide one.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA

=cut

1;
